package example;

import android.app.Activity;
import android.app.Application;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.widget.TextView;
import android.widget.Toast;

import com.qmuiteam.qmui.util.QMUIDisplayHelper;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import example.fragment.functions.dialog.DialogMenuFragment;
import example.fragment.functions.file.FileMenuFragment;
import example.fragment.functions.form.FormFragment;
import example.fragment.functions.image.ImageMenuFragment;
import example.fragment.functions.layout.LayoutMenuFragment;
import example.fragment.functions.notice.NoticeMenuFragment;
import example.fragment.functions.util.UtilMenuFragment;
import example.fragment.functions.widget.WidgetMenuFragment;
import xyz.wbsite.base.BaseSPAFragment;
import xyz.wbsite.util.DataBaseUtil;
import xyz.wbsite.util.DialogUtil;
import xyz.wbsite.wbui.R;


/**
 * Application 入口。
 */
public class WBUIApplication extends Application implements Thread.UncaughtExceptionHandler {

    private static WBUIApplication instance;
    private WeakReference<Activity> currentActivity;

    private DataBaseUtil db;
    private Map<String, Object> map;
    private IconManager iconManager;
    private Toast toast;

    public static WBUIApplication getInstance() {
        return instance;
    }

    public static Map getMap() {
        return instance.map;
    }

    public DataBaseUtil getDb() {
        return db;
    }

    public IconManager getIconManager() {
        return iconManager;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        instance = this;
        initExceptionHandler();
        initMap();
        initLife();
        initIconMgr();
    }

    public void lozyInit() {
        initDB();
    }

    private void initDB() {
        db = new DataBaseUtil(this, new DataBaseUtil.Register() {
            @Override
            public List<Class> run() {
                return new ArrayList<>();
            }
        });
    }

    private void initMap() {
        map = new HashMap();
    }

    private void initIconMgr() {
        iconManager = new IconManager();
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_widget, "UI组件", BaseSPAFragment.newInstance(WidgetMenuFragment.class)));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_layout, "容器组件", BaseSPAFragment.newInstance(LayoutMenuFragment.class)));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_form, "表单组件", BaseSPAFragment.newInstance(FormFragment.class)));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_image, "图片处理", BaseSPAFragment.newInstance(ImageMenuFragment.class)));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_file, "文件选择", BaseSPAFragment.newInstance(FileMenuFragment.class)));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_dialog, "对话框", BaseSPAFragment.newInstance(DialogMenuFragment.class)));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_util, "工具类", BaseSPAFragment.newInstance(UtilMenuFragment.class)));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_qrcode, "二维码", null));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_restart, "重启", null));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_upgrade, "更新版本", null));
        iconManager.addIcon(iconManager.new Icon(R.drawable.icon_fun_notice, "消息通知", BaseSPAFragment.newInstance(NoticeMenuFragment.class)));
    }

    private void initLife() {
        registerActivityLifecycleCallbacks(new ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle savedInstanceState) {

            }

            @Override
            public void onActivityStarted(Activity activity) {

            }

            @Override
            public void onActivityResumed(Activity activity) {
                currentActivity = new WeakReference<Activity>(activity);
            }

            @Override
            public void onActivityPaused(Activity activity) {

            }

            @Override
            public void onActivityStopped(Activity activity) {

            }

            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

            }

            @Override
            public void onActivityDestroyed(Activity activity) {

            }
        });
    }

    private void initExceptionHandler() {
        // 程序的默认处理器
        Thread.setDefaultUncaughtExceptionHandler(this);
    }

    // 判断是否安装插件
    public boolean hasInstallPlugin(String packageName) {
        try {
            PackageInfo info = getPackageManager().getPackageInfo(packageName, 0);
            return (info != null);
        } catch (PackageManager.NameNotFoundException e) {
        }
        return false;
    }

    /**
     * 展示Toast消息。
     *
     * @param message 消息内容
     */
    public synchronized void showToast(final String message) {
        if (toast == null) {
            toast = new Toast(this);
            toast.setGravity(Gravity.CENTER, 0, 0);
            TextView textView = new TextView(this);
            textView.setTextColor(Color.parseColor("#ffffff"));
            int padding = QMUIDisplayHelper.dp2px(this, 20);
            textView.setPadding(padding, padding, padding, padding);
            textView.setBackgroundColor(Color.parseColor("#aa131313"));
            toast.setView(textView);
        }
        ((TextView) toast.getView()).setText(message);
        toast.show();
    }

    @Override
    public void uncaughtException(Thread thread, Throwable throwable) {
        Log.e("error--->", "error : ", throwable);
        exit();
    }

    public void exit() {
        DialogUtil.showCheckboxMessage(currentActivity.get(), "请确保工作内容已保存!", "是否确认退出程序", new DialogUtil.MessageListener() {
            @Override
            public void onCallback(boolean positive) {
                if (positive) {
                    android.os.Process.killProcess(android.os.Process.myPid());
                }
            }
        });
    }
}
