package xyz.wbsite.activity;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.Matrix;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ImageView;

import com.google.zxing.BinaryBitmap;
import com.google.zxing.MultiFormatReader;
import com.google.zxing.RGBLuminanceSource;
import com.google.zxing.Reader;
import com.google.zxing.Result;
import com.google.zxing.client.android.Intents;
import com.google.zxing.common.HybridBinarizer;
import com.journeyapps.barcodescanner.CaptureManager;
import com.journeyapps.barcodescanner.DecoratedBarcodeView;
import com.qmuiteam.qmui.util.QMUIDisplayHelper;

import xyz.wbsite.base.BaseActivity;
import xyz.wbsite.base.IActivityResult;
import xyz.wbsite.wbui.R;

/**
 * 自定义二维码扫描布局Activity，由二维码扫描入口{@link QRcodeActivity}中指定自定义布局
 */
public class QRcodeScanActivity extends BaseActivity implements DecoratedBarcodeView.TorchListener {

    private CaptureManager capture;
    private DecoratedBarcodeView barcodeScannerView;
    private ImageView imgSwitchFlashlight;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.ui_activity_qrcode);

        QMUIDisplayHelper.setFullScreen(this);

        barcodeScannerView = (DecoratedBarcodeView) findViewById(R.id.zxing_barcode_scanner);
        barcodeScannerView.setTorchListener(this);

        imgSwitchFlashlight = (ImageView) findViewById(R.id.switch_flashlight);
        // if the device does not have flashlight in its camera,
        // then remove the switch flashlight button...
        imgSwitchFlashlight.setTag(R.drawable.ic_flash_off);
        if (!hasFlash()) {
            imgSwitchFlashlight.setVisibility(View.GONE);
        }

        capture = new CaptureManager(this, barcodeScannerView);
        capture.initializeFromIntent(getIntent(), savedInstanceState);
        capture.decode();
    }

    @Override
    protected void onResume() {
        super.onResume();
        capture.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        capture.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        capture.onDestroy();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        capture.onSaveInstanceState(outState);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        return barcodeScannerView.onKeyDown(keyCode, event) || super.onKeyDown(keyCode, event);
    }

    private boolean hasFlash() {
        return getApplicationContext().getPackageManager()
                .hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH);
    }

    public void switchFlashlight(View view) {
        int tag = (int) imgSwitchFlashlight.getTag();
        if (R.drawable.ic_flash_off == tag) {
            barcodeScannerView.setTorchOn();
        } else {
            barcodeScannerView.setTorchOff();
        }
    }

    public void pickImage(View view) {
        Intent intent = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
        startForResult(intent, new IActivityResult() {
            @Override
            public void onResult(int resultCode, Intent data) {
                if (Activity.RESULT_OK == resultCode && null != data) {
                    Uri sourceUri = data.getData();
                    try {
                        // 下面这句话可以通过URi获取到文件的bitmap
                        Bitmap bitmap = MediaStore.Images.Media.getBitmap(QRcodeScanActivity.this.getContentResolver(), sourceUri);

                        // 在这里我用到的 getSmallerBitmap 非常重要，下面就要说到
                        bitmap = getSmallerBitmap(bitmap);

                        // 获取bitmap的宽高，像素矩阵
                        int width = bitmap.getWidth();
                        int height = bitmap.getHeight();
                        int[] pixels = new int[width * height];
                        bitmap.getPixels(pixels, 0, width, 0, 0, width, height);

                        // 最新的库中，RGBLuminanceSource 的构造器参数不只是bitmap了
                        RGBLuminanceSource source = new RGBLuminanceSource(width, height, pixels);
                        BinaryBitmap binaryBitmap = new BinaryBitmap(new HybridBinarizer(source));
                        Reader reader = new MultiFormatReader();
                        Result result = null;

                        // 尝试解析此bitmap，！！注意！！ 这个部分一定写到外层的try之中，因为只有在bitmap获取到之后才能解析。写外部可能会有异步的问题。（开始解析时bitmap为空）
                        result = reader.decode(binaryBitmap);
                        Intent intent = new Intent();
                        intent.putExtra(Intents.Scan.RESULT, result.getText());
                        setResult(Activity.RESULT_OK, intent);
                        finish();
                    } catch (Exception e) {
                        setResult(Activity.RESULT_CANCELED);
                        finish();
                    }
                } else {
                    setResult(Activity.RESULT_CANCELED);
                    finish();
                }
            }
        });
    }

    public static Bitmap getSmallerBitmap(Bitmap bitmap) {
        int size = bitmap.getWidth() * bitmap.getHeight() / 160000;
        if (size <= 1) return bitmap; // 如果小于
        else {
            Matrix matrix = new Matrix();
            matrix.postScale((float) (1 / Math.sqrt(size)), (float) (1 / Math.sqrt(size)));
            Bitmap resizeBitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
            return resizeBitmap;
        }
    }

    @Override
    public void onTorchOn() {
        imgSwitchFlashlight.setImageResource(R.drawable.ic_flash_on);
        imgSwitchFlashlight.setTag(R.drawable.ic_flash_on);
    }

    @Override
    public void onTorchOff() {
        imgSwitchFlashlight.setImageResource(R.drawable.ic_flash_off);
        imgSwitchFlashlight.setTag(R.drawable.ic_flash_off);
    }
}