package xyz.wbsite.activity;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;

import com.jph.takephoto.app.TakePhoto;
import com.jph.takephoto.compress.CompressConfig;
import com.jph.takephoto.model.CropOptions;
import com.jph.takephoto.model.TImage;
import com.jph.takephoto.model.TResult;

import java.io.File;

import xyz.wbsite.base.Consant;
import xyz.wbsite.util.StorageUtil;

public class TakePhotoActivity extends com.jph.takephoto.app.TakePhotoActivity {

    public static final String POSITION_PARAM = "position_param";
    public static final String POSITION_CAPTURE = "capture";
    public static final String POSITION_GALLERY = "gallery";
    public static final String POSITION_DOCUMENTS = "documents";

    private CropOptions cropOptions;  //裁剪参数
    private CompressConfig compressConfig;  //压缩参数
    private TakePhoto takePhoto;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        takePhoto = getTakePhoto();
        configCompress();
        configCrop();
        Intent intent = getIntent();
        String stringExtra = intent.getStringExtra(POSITION_PARAM);
        if (POSITION_CAPTURE.equals(stringExtra)) {
            takePhoto.onPickFromCapture(getImageCropUri());
        } else if (POSITION_GALLERY.equals(stringExtra)) {
            takePhoto.onPickFromGallery();
        } else if (POSITION_DOCUMENTS.equals(stringExtra)) {
            takePhoto.onPickFromDocuments();
        }
    }

    /**
     * 获得照片的输出保存Uri
     *
     * @return
     */
    private Uri getImageCropUri() {
        File file = new File(StorageUtil.getLocalDir(this, Consant.DIR_IMG), System.currentTimeMillis() + ".jpg");
        if (!file.getParentFile().exists()) file.getParentFile().mkdirs();
        return Uri.fromFile(file);
    }

    /**
     * 设置压缩参数
     */
    private void configCompress() {//压缩配置
        CompressConfig config = new CompressConfig.Builder().setMaxSize(512 * 1024)
                .setMaxPixel(800)
                .enableReserveRaw(false)//拍照压缩后是否显示原图
                .create();
        takePhoto.onEnableCompress(config, false);//是否显示进度条
    }

    /**
     * 设置压缩参数
     */
    private void configCrop() {//裁剪参数
        cropOptions = new CropOptions.Builder().setAspectX(240).setAspectY(320).setWithOwnCrop(true).create();
    }

    @Override
    public void takeSuccess(TResult result) {
        TImage image = result.getImage();
        Intent intent = new Intent();
        intent.setData(Uri.fromFile(new File(image.getCompressPath() != null ? image.getCompressPath() : image.getOriginalPath())));
        setResult(RESULT_OK, intent);
        finish();
    }

    @Override
    public void takeFail(TResult result, String msg) {
        super.takeFail(result, msg);
        setResult(RESULT_CANCELED);
        finish();
    }

    @Override
    public void takeCancel() {
        super.takeCancel();
        setResult(RESULT_CANCELED);
        finish();
    }
}
