package xyz.wbsite.base;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.FragmentActivity;
import android.view.LayoutInflater;
import android.view.View;

import com.qmuiteam.qmui.arch.QMUIFragment;
import com.qmuiteam.qmui.util.QMUIDisplayHelper;

import java.lang.reflect.InvocationTargetException;
import java.util.Hashtable;

import butterknife.ButterKnife;
import butterknife.Unbinder;

public abstract class BaseSPAFragment extends QMUIFragment {
    private Unbinder unbinder;
    protected Handler handler = new Handler();
    private Hashtable<Integer, IFragmentResult> mFragmentResult = new Hashtable<>();
    private int mRequestCode = 1;

    public void startForResult(BaseSPAFragment intent, IFragmentResult result) {
        int requestCode = mRequestCode++;
        mFragmentResult.put(requestCode, result);
        startFragmentForResult(intent, requestCode);
    }

    @Override
    protected void onFragmentResult(int requestCode, int resultCode, Intent data) {
        super.onFragmentResult(requestCode, resultCode, data);
        if (mFragmentResult.containsKey(requestCode)) {
            IFragmentResult l = mFragmentResult.remove(requestCode);
            if (l != null) {
                l.onResult(resultCode, data);
            }
        }
    }

    @Override
    protected int backViewInitOffset() {
        return QMUIDisplayHelper.dp2px(getContext(), 100);
    }

    protected abstract int getFragmnetLayout();

    @Override
    protected View onCreateView() {
        View inflate = LayoutInflater.from(getActivity()).inflate(getFragmnetLayout(), null);
        unbinder = ButterKnife.bind(this, inflate);
        onViewInit();
        onDataInit(getArguments());
        return inflate;
    }

    /**
     * 初始化View，事件等
     */
    protected abstract void onViewInit();

    /**
     * 数据保存
     *
     * @param data Bundle对象
     */
    protected abstract void onDataSave(Bundle data);

    /**
     * 数据初始化/恢复
     *
     * @param data Bundle对象
     */
    protected abstract void onDataInit(Bundle data);

    public void showLoading() {
        BaseSPAActivity activity = (BaseSPAActivity) getActivity();
        activity.showLoading();
    }

    public void closeLoading() {
        BaseSPAActivity activity = (BaseSPAActivity) getActivity();
        activity.dismissLoading();
    }

    @Override
    public void onSaveInstanceState(@NonNull Bundle outState) {
        super.onSaveInstanceState(outState);
        onDataSave(outState);
    }

    @Override
    public void onViewStateRestored(@Nullable Bundle savedInstanceState) {
        super.onViewStateRestored(savedInstanceState);
        onDataInit(getArguments());
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onDestroyView() {
        onDataSave(getArguments());
        super.onDestroyView();
    }

    @Override
    public void onDetach() {
        super.onDetach();
        if (unbinder != null) {
            unbinder.unbind();
            unbinder = null;
        }
    }

    public void startForResult(Intent intent, IActivityResult activityResult) {
        FragmentActivity activity = getActivity();
        if (activity instanceof BaseSPAActivity) {
            BaseSPAActivity baseSPAActivity = (BaseSPAActivity) activity;
            baseSPAActivity.startForResult(intent, activityResult);
        }
    }

    public static BaseSPAFragment newInstance(@NonNull Class<? extends BaseSPAFragment> fragmentClass) {
        BaseSPAFragment fragment = null;
        try {
            fragment = fragmentClass.getConstructor().newInstance();
            fragment.setArguments(new Bundle());
        } catch (java.lang.InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }

        return fragment;
    }

    @Override
    protected boolean canDragBack() {
        return false;
    }
}