package xyz.wbsite.util;

import android.content.Context;
import android.content.DialogInterface;

import com.qmuiteam.qmui.widget.dialog.QMUIDialog;
import com.qmuiteam.qmui.widget.dialog.QMUIDialogAction;


public class DialogUtil {

    public static void showMessage(Context context, String title, String message) {
        new QMUIDialog.MessageDialogBuilder(context)
                .setTitle(title)
                .setMessage(message)
                .addAction("确认", new QMUIDialogAction.ActionListener() {
                    @Override
                    public void onClick(QMUIDialog dialog, int index) {
                        dialog.dismiss();
                    }
                })
                .create(com.qmuiteam.qmui.R.style.QMUI_Dialog).show();
    }

    public static void showMessage(Context context, String title, String message, final MessageListener messageListener) {
        new QMUIDialog.MessageDialogBuilder(context)
                .setTitle(title)
                .setMessage(message)
                .addAction("取消", new QMUIDialogAction.ActionListener() {
                    @Override
                    public void onClick(QMUIDialog dialog, int index) {
                        dialog.dismiss();
                    }
                })
                .addAction("确定", new QMUIDialogAction.ActionListener() {
                    @Override
                    public void onClick(QMUIDialog dialog, int index) {
                        dialog.dismiss();
                        messageListener.onCallback(true);
                    }
                })
                .create(com.qmuiteam.qmui.R.style.QMUI_Dialog).show();
    }

    public static void showCheckboxMessage(Context context, String title, String message, final MessageListener messageListener) {
        final QMUIDialog.CheckBoxMessageDialogBuilder builder = new QMUIDialog.CheckBoxMessageDialogBuilder(context);
        builder.setTitle(title);
        builder.setMessage(message);
        builder.setChecked(true);
        builder.addAction("取消", new QMUIDialogAction.ActionListener() {
            @Override
            public void onClick(QMUIDialog dialog, int index) {
                dialog.dismiss();
            }
        });
        builder.addAction(0, "确定", new QMUIDialogAction.ActionListener() {
            @Override
            public void onClick(QMUIDialog dialog, int index) {
                dialog.dismiss();
                messageListener.onCallback(builder.isChecked());
            }
        });
        builder.create(com.qmuiteam.qmui.R.style.QMUI_Dialog);
        builder.show();
    }

    public static void showOption(Context context, String[] options, final OptionListener optionListener) {
        new QMUIDialog.MenuDialogBuilder(context)
                .addItems(options, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.dismiss();
                        optionListener.onCallback(which);
                    }
                })
                .create(com.qmuiteam.qmui.R.style.QMUI_Dialog)
                .show();
    }

    public static void showMultiOption(Context context, String[] options, int[] checked, final MultiOptionListener onClickListener) {
        final QMUIDialog.MultiCheckableDialogBuilder builder = new QMUIDialog.MultiCheckableDialogBuilder(context)
                .setCheckedItems(checked)
                .addItems(options, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {

                        System.out.println();

                    }
                });
        builder.addAction("取消", new QMUIDialogAction.ActionListener() {
            @Override
            public void onClick(QMUIDialog dialog, int index) {
                dialog.dismiss();
            }
        });
        builder.addAction("提交", new QMUIDialogAction.ActionListener() {
            @Override
            public void onClick(QMUIDialog dialog, int index) {
                dialog.dismiss();
                int[] checkedItemIndexes = builder.getCheckedItemIndexes();
                onClickListener.onCallback(checkedItemIndexes);
            }
        });
        builder.create(com.qmuiteam.qmui.R.style.QMUI_Dialog);
        builder.show();
    }

    public interface MessageListener {
        void onCallback(boolean positive);
    }

    public interface OptionListener {
        void onCallback(int selected);
    }

    public interface MultiOptionListener {
        void onCallback(int[] selected);
    }
}