package xyz.wbsite.wbui.layout;

import android.content.Context;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.View;
import android.widget.LinearLayout;

import xyz.wbsite.wbui.other.WBUILoadingProgressBar;

public class WBUIRefreshLayout extends WBUIPaternalLayout {

    private PullRefreshListenter pullRefreshListenter;
    private PushRefreshListenter pushRefreshListenter;

    public WBUIRefreshLayout(Context context) {
        super(context);
        init();
    }

    public WBUIRefreshLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    private void init() {
        this.setPullViewBuilder(new WBUIPaternalLayout.IPullViewBuilder() {
            WBUILoadingProgressBar progressBar;

            @Override
            public View createView() {
                LinearLayout linearLayout = new LinearLayout(getContext());
                linearLayout.setGravity(Gravity.CENTER);
                progressBar = new WBUILoadingProgressBar(getContext());
                linearLayout.addView(progressBar);
                return linearLayout;
            }

            @Override
            public void onChange(View view, int currentHeight, int pullHeight, boolean isNearHeight) {
                if (currentHeight > pullHeight) {
                    progressBar.setProgress(1);
                } else {
                    progressBar.setProgress(currentHeight * 1.0f / pullHeight);
                }
            }

            @Override
            public void onAction(View view, int pullHeight, WBUIPaternalLayout layout) {
                progressBar.startAnimation();
                if (pullRefreshListenter != null) {
                    pullRefreshListenter.onRefresh();
                }
                WBUIRefreshLayout.this.finish();
            }

            @Override
            public void onFinish(View view) {
                progressBar.setProgress(0);
                progressBar.stopAnimation();
            }
        });
        this.setPushViewBuilder(new WBUIPaternalLayout.IPushViewBuilder() {
            WBUILoadingProgressBar progressBar;

            @Override
            public View createView() {
                LinearLayout linearLayout = new LinearLayout(getContext());
                linearLayout.setGravity(Gravity.CENTER);
                progressBar = new WBUILoadingProgressBar(getContext());
                linearLayout.addView(linearLayout);
                return linearLayout;
            }

            @Override
            public void onChange(View view, int currentHeight, int pullHeight, boolean isNearHeight) {
                if (currentHeight > pullHeight) {
                    progressBar.setProgress(1);
                } else {
                    progressBar.setProgress(currentHeight * 1.0f / pullHeight);
                }
            }

            @Override
            public void onAction(View view, int pullHeight, WBUIPaternalLayout layout) {
                progressBar.startAnimation();
                if (pushRefreshListenter != null) {
                    pushRefreshListenter.onRefresh();
                }
                WBUIRefreshLayout.this.finish();
            }

            @Override
            public void onFinish(View view) {
                progressBar.setProgress(0);
                progressBar.stopAnimation();
            }
        });
    }

    public PullRefreshListenter getPullRefreshListenter() {
        return pullRefreshListenter;
    }

    public void setPullRefreshListenter(PullRefreshListenter pullRefreshListenter) {
        this.pullRefreshListenter = pullRefreshListenter;
    }

    public PushRefreshListenter getPushRefreshListenter() {
        return pushRefreshListenter;
    }

    public void setPushRefreshListenter(PushRefreshListenter pushRefreshListenter) {
        this.pushRefreshListenter = pushRefreshListenter;
    }

    public interface PullRefreshListenter {
        void onRefresh();
    }

    public interface PushRefreshListenter {
        void onRefresh();
    }
}