package xyz.wbsite.wbui.layout;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.View;
import android.widget.RelativeLayout;

import xyz.wbsite.wbui.R;

/**
 * 固定长宽布局
 */
public class WBUIScaleLayout extends RelativeLayout {

    private int weight_width = 1;
    private int weight_height = 1;
    private int mWidth;
    private int mHeight;

    private View targetView;

    public WBUIScaleLayout(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
    }

    public WBUIScaleLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        setWillNotDraw(false);
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.ScaleView);
        weight_width = typedArray.getInt(R.styleable.ScaleView_weight_width, weight_width);
        weight_height = typedArray.getInt(R.styleable.ScaleView_weight_height, weight_height);
    }

    public WBUIScaleLayout(Context context) {
        super(context);
    }

    private void ensureView() {
        int childCount = getChildCount();
        if (childCount > 1) {
            throw new IllegalStateException("WBUISquareLayout can host only one child");
        } else if (childCount == 1) {
            targetView = getChildAt(0);
        }
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        int wMode = MeasureSpec.getMode(widthMeasureSpec);
        int wSize = MeasureSpec.getSize(widthMeasureSpec);
        int hMode = MeasureSpec.getMode(heightMeasureSpec);
        int hSize = MeasureSpec.getSize(heightMeasureSpec);

        if (wMode == MeasureSpec.EXACTLY && hMode == MeasureSpec.EXACTLY) {
            mWidth = wSize;
            mHeight = hSize;
        } else if (wMode == MeasureSpec.EXACTLY) {
            mWidth = wSize;
            mHeight = wSize * weight_height / weight_width;
        } else if (hMode == MeasureSpec.EXACTLY) {
            mWidth = hSize * weight_width / weight_height;
            mHeight = hSize;
        } else {
            measureChildren(widthMeasureSpec, heightMeasureSpec);
            ensureView();
            if (targetView != null) {
                int measuredWidth = targetView.getMeasuredWidth();
                int measuredHeight = targetView.getMeasuredHeight();
                mWidth = mHeight = Math.max(measuredWidth, measuredHeight);
            }
        }
        setMeasuredDimension(mWidth, mHeight);
        measureChildren(MeasureSpec.makeMeasureSpec(mWidth, MeasureSpec.AT_MOST), MeasureSpec.makeMeasureSpec(mHeight, MeasureSpec.AT_MOST));
        super.onMeasure(MeasureSpec.makeMeasureSpec(mWidth, MeasureSpec.EXACTLY), MeasureSpec.makeMeasureSpec(mHeight, MeasureSpec.EXACTLY));
    }
}