package xyz.wbsite.wbui.widget.button;

import android.content.Context;
import android.content.res.ColorStateList;
import android.content.res.TypedArray;
import android.graphics.Color;
import android.graphics.RectF;
import android.graphics.drawable.ShapeDrawable;
import android.graphics.drawable.StateListDrawable;
import android.graphics.drawable.shapes.RoundRectShape;
import android.util.AttributeSet;

import xyz.wbsite.wbui.R;

/**
 * 封装好的Button
 */
public class WBUIButton extends android.support.v7.widget.AppCompatButton {
    private int mColor;
    private float mRadius;
    private float mBorderWidth;

    public WBUIButton(Context context) {
        super(context);
        init(null, 0);
    }

    public WBUIButton(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(attrs, 0);
    }

    public WBUIButton(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init(attrs, defStyle);
    }

    private void init(AttributeSet attrs, int defStyle) {
        TypedArray a = getContext().obtainStyledAttributes(attrs, R.styleable.WBUIButton, defStyle, 0);
        mColor = this.getTextColors().getDefaultColor();
        mRadius = a.getDimensionPixelSize(R.styleable.WBUIButton_border_radius, 0);
        mBorderWidth = a.getDimensionPixelSize(R.styleable.WBUIButton_border_width, 1);
        a.recycle();

        StateListDrawable stateListDrawable = new StateListDrawable();
        stateListDrawable.setEnterFadeDuration(200);
        stateListDrawable.setExitFadeDuration(200);

        //不可按时的drawable
        {
            RoundRectShape roundRectShape = new RoundRectShape(
                    new float[]{mRadius, mRadius, mRadius, mRadius, mRadius, mRadius, mRadius, mRadius},
                    new RectF(mBorderWidth, mBorderWidth, mBorderWidth, mBorderWidth),
                    new float[]{mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth}
            );
            ShapeDrawable shapeDrawable = new ShapeDrawable(roundRectShape);
            shapeDrawable.getPaint().setColor(Color.parseColor("#DDDDDD"));
            stateListDrawable.addState(new int[]{-android.R.attr.state_enabled}, shapeDrawable);
        }
        //正常的drawable
        {
            RoundRectShape roundRectShape = new RoundRectShape(
                    new float[]{mRadius, mRadius, mRadius, mRadius, mRadius, mRadius, mRadius, mRadius},
                    new RectF(mBorderWidth, mBorderWidth, mBorderWidth, mBorderWidth),
                    new float[]{mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth, mRadius - mBorderWidth}
            );
            ShapeDrawable shapeDrawable = new ShapeDrawable(roundRectShape);
            shapeDrawable.getPaint().setColor(mColor);
            stateListDrawable.addState(new int[]{-android.R.attr.state_pressed}, shapeDrawable);
        }
        //按下去的drawable
        {
            RoundRectShape roundRectShape = new RoundRectShape(
                    new float[]{mRadius, mRadius, mRadius, mRadius, mRadius, mRadius, mRadius, mRadius},
                    null,
                    null
            );
            ShapeDrawable shapeDrawable = new ShapeDrawable(roundRectShape);
            shapeDrawable.getPaint().setColor(mColor);
            stateListDrawable.addState(new int[]{android.R.attr.state_pressed}, shapeDrawable);
        }
        this.setBackground(stateListDrawable);

        //各状态对应的字体颜色
        int[][] satate = {
                {-android.R.attr.state_enabled},
                {-android.R.attr.state_pressed},
                {android.R.attr.state_pressed}
        };
        int[] colors = {Color.parseColor("#DDDDDD"), mColor, Color.WHITE};

        ColorStateList colorStateList = new ColorStateList(satate, colors);
        this.setTextColor(colorStateList);
    }
}
