package xyz.wbsite.wbui.widget.button;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.drawable.AnimationDrawable;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.StateListDrawable;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.TypedValue;

import xyz.wbsite.wbui.R;

public class WBUIRadioButton extends android.support.v7.widget.AppCompatRadioButton {
    private int mRadioWidth;
    private int mColor;
    private AnimationDrawable animationDrawable;

    public WBUIRadioButton(Context context) {
        super(context);
        init(null, 0);
    }

    public WBUIRadioButton(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(attrs, 0);
    }

    public WBUIRadioButton(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init(attrs, defStyle);
    }

    private void init(AttributeSet attrs, int defStyle) {
        DisplayMetrics metrics = getResources().getDisplayMetrics();
        TypedArray a = getContext().obtainStyledAttributes(attrs, R.styleable.WBUIRadioButton, defStyle, 0);
        mRadioWidth = a.getDimensionPixelSize(
                R.styleable.WBUIRadioButton_radio_width,
                (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 24, metrics));
        mColor = a.getColor(R.styleable.WBUIRadioButton_radio_color, getResources().getColor(R.color.colorPrimary));
        a.recycle();

        StateListDrawable stateListDrawable = new StateListDrawable();
        stateListDrawable.setEnterFadeDuration(300);
        stateListDrawable.setExitFadeDuration(200);

        {//未选中状态
            Bitmap unchecked = Bitmap.createBitmap(mRadioWidth, mRadioWidth, Bitmap.Config.ARGB_8888);
            Canvas canvas = new Canvas(unchecked);
            Paint paint = new Paint();
            paint.setColor(Color.parseColor("#999999"));
            paint.setStyle(Paint.Style.STROKE);
            paint.setStrokeWidth(TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 2, metrics));
            canvas.drawRect(
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 4, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 4, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 20, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 20, metrics), paint);

            stateListDrawable.addState(new int[]{-android.R.attr.state_checked}, new BitmapDrawable(getResources(), unchecked));
        }

        {//选中状态
            animationDrawable = new AnimationDrawable();
            animationDrawable.setOneShot(true);

            Matrix matrix = new Matrix();
            Paint paint = new Paint();
            int scale = 100;
            while (scale > 50) {
                matrix.setScale(scale / 100f, scale / 100f,
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 12, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 12, metrics));
                Bitmap checked = Bitmap.createBitmap(mRadioWidth, mRadioWidth, Bitmap.Config.ARGB_8888);
                Canvas canvas = new Canvas(checked);
                paint.setColor(mColor);
                paint.setStyle(Paint.Style.STROKE);
                paint.setStrokeWidth(TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 2, metrics));
                canvas.drawRect(
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 4, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 4, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 20, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 20, metrics), paint);
                paint.setStyle(Paint.Style.FILL);
                canvas.setMatrix(matrix);
                canvas.drawRect(
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 4, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 4, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 20, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 20, metrics), paint);

                animationDrawable.addFrame(new BitmapDrawable(getResources(), checked), 30);
                scale -= 5;
            }

            stateListDrawable.addState(new int[]{android.R.attr.state_checked}, animationDrawable);
        }

        this.setButtonDrawable(stateListDrawable);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        int measuredHeight = getMeasuredHeight();
        int offsetY = (measuredHeight - getPaddingTop() - getPaddingBottom() - mRadioWidth) / 2 + getPaddingTop();
        canvas.translate(0, offsetY);
        super.onDraw(canvas);
    }

    @Override
    public void setChecked(boolean checked) {
        super.setChecked(checked);
        if (checked && animationDrawable != null) {
            animationDrawable.start();
        }
    }
}