package xyz.wbsite.wbui.widget.button;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.AnimationDrawable;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.StateListDrawable;
import android.os.Build;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.TypedValue;

import xyz.wbsite.wbui.R;

public class WBUISwitchButton extends android.support.v7.widget.AppCompatCheckBox {
    private int mRadioWidth;
    private int mRadioHeight;
    private Paint paint = new Paint();
    private AnimationDrawable animationDrawable;

    public WBUISwitchButton(Context context) {
        super(context);
        init(null, 0);
    }

    public WBUISwitchButton(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(attrs, 0);
    }

    public WBUISwitchButton(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init(attrs, defStyle);
    }

    private void init(AttributeSet attrs, int defStyle) {
        DisplayMetrics metrics = getResources().getDisplayMetrics();
        mRadioWidth = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 40, metrics);
        mRadioHeight = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 26, metrics);

        StateListDrawable stateListDrawable = new StateListDrawable();
        stateListDrawable.setEnterFadeDuration(300);
        stateListDrawable.setExitFadeDuration(200);

        {//未选中状态
            Bitmap unchecked = Bitmap.createBitmap(mRadioWidth, mRadioWidth, Bitmap.Config.ARGB_8888);
            Canvas canvas = new Canvas(unchecked);
            paint.setColor(getResources().getColor(R.color.colorSwitchDisEnable));
            paint.setStyle(Paint.Style.FILL);
            canvas.drawRoundRect(
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 40, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 26, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                    paint);
            paint.setStrokeWidth(1);
            paint.setColor(getResources().getColor(R.color.colorSwitchBorder));
            paint.setStyle(Paint.Style.STROKE);
            canvas.drawRoundRect(
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 40, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 26, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                    paint);
            paint.setStyle(Paint.Style.FILL);
            paint.setColor(getResources().getColor(R.color.colorSwitchButton));
            canvas.drawRoundRect(
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 1, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 1, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 25, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 25, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 12, metrics),
                    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 12, metrics),
                    paint);
            stateListDrawable.addState(new int[]{-android.R.attr.state_checked}, new BitmapDrawable(getResources(), unchecked));
        }

        {//选中状态
            animationDrawable = new AnimationDrawable();
            animationDrawable.setOneShot(true);

            Paint paint = new Paint();
            for (int shift = 0; shift < 15; shift++) {
                Bitmap checked = Bitmap.createBitmap(mRadioWidth, mRadioWidth, Bitmap.Config.ARGB_8888);
                Canvas canvas = new Canvas(checked);
                paint.setColor(getResources().getColor(R.color.colorSwitchDisEnable));
                paint.setStyle(Paint.Style.FILL);
                canvas.drawRoundRect(
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 40, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 26, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                        paint);
                paint.setColor(getResources().getColor(R.color.colorSwitchEnable));
                canvas.drawRoundRect(
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 26 + shift, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 26, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                        paint);
                paint.setStrokeWidth(1);
                paint.setColor(getResources().getColor(R.color.colorSwitchBorder));
                paint.setStyle(Paint.Style.STROKE);
                canvas.drawRoundRect(
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 0, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 40, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 26, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 13, metrics),
                        paint);
                paint.setStyle(Paint.Style.FILL);
                paint.setColor(getResources().getColor(R.color.colorSwitchButton));
                canvas.translate(TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, shift, metrics), 0);
                canvas.drawRoundRect(
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 1, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 1, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 25, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 25, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 12, metrics),
                        TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 12, metrics),
                        paint);
                animationDrawable.addFrame(new BitmapDrawable(getResources(), checked), 16);
            }

            stateListDrawable.addState(new int[]{android.R.attr.state_checked}, animationDrawable);
        }

        this.setButtonDrawable(stateListDrawable);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            this.getButtonDrawable().setBounds(0, 0, mRadioWidth, mRadioHeight);
        }
    }

    @Override
    protected void onDraw(Canvas canvas) {
        int measuredWidth = getMeasuredWidth();
        int measuredHeight = getMeasuredHeight();
        int offsetX = (measuredWidth - getPaddingLeft() - getPaddingRight() - mRadioWidth) / 2 + getPaddingLeft();
        int offsetY = (measuredHeight - getPaddingTop() - getPaddingBottom() - mRadioHeight) / 2 + getPaddingTop();
        canvas.translate(offsetX, offsetY);
        super.onDraw(canvas);
    }

    @Override
    public void setChecked(boolean checked) {
        super.setChecked(checked);
        if (checked && animationDrawable != null) {
            animationDrawable.start();
        }
    }
}