package xyz.wbsite.wbui.widget.form;

import android.content.Context;
import android.content.res.TypedArray;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.view.View;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import cn.qqtheme.framework.picker.DateTimePicker;
import xyz.wbsite.base.BaseSPAActivity;
import xyz.wbsite.wbui.R;

public class DATETIMEFormInputView extends FormInputView {
    private Date currentDate;
    private SimpleDateFormat dateFormat;

    private DateTimePicker datePicker;

    public DATETIMEFormInputView(Context context) {
        super(context);
    }

    public DATETIMEFormInputView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    @Override
    protected void init(AttributeSet attrs) {
        super.init(attrs);
        // 获取自定义属性
        TypedArray a = getContext().obtainStyledAttributes(attrs, R.styleable.FormView);
        String format_ = a.getString(R.styleable.FormView_format);
        if (format_ == null) {
            format_ = "yyyy-MM-dd HH:mm:ss";
        }
        dateFormat = new SimpleDateFormat(format_, Locale.CHINA);
        a.recycle();
        //初始化InputView
        edtInput.setEnabled(true);
        edtInput.setFocusable(false);
        edtInput.setKeyListener(null);
        edtInput.setHint("请选择日期时间");
        edtInput.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                onShowPicker();
            }
        });
    }

    private void onShowPicker() {
        if (currentDate == null) {
            currentDate = new Date();
        }

        Calendar instance = Calendar.getInstance();
        instance.setTime(currentDate);

        //初始化日期选择器
        datePicker = new DateTimePicker((BaseSPAActivity) getContext(), DateTimePicker.HOUR_24);
        datePicker.setDateRangeStart(instance.get(Calendar.YEAR) - 80, 1, 1);
        datePicker.setDateRangeEnd(instance.get(Calendar.YEAR) + 20, 12, 31);
        datePicker.setSelectedItem(
                instance.get(Calendar.YEAR),
                instance.get(Calendar.MONTH) + 1,
                instance.get(Calendar.DAY_OF_MONTH),
                instance.get(Calendar.HOUR_OF_DAY),
                instance.get(Calendar.MINUTE));
        datePicker.setOnDateTimePickListener(new DateTimePicker.OnYearMonthDayTimePickListener() {

            @Override
            public void onDateTimePicked(String year, String month, String day, String hour, String minute) {
                Calendar instance = Calendar.getInstance();
                instance.setTime(currentDate);
                instance.set(
                        Integer.parseInt(year),
                        Integer.parseInt(month) - 1,
                        Integer.parseInt(day),
                        Integer.parseInt(hour),
                        Integer.parseInt(minute),
                        0);
                currentDate = instance.getTime();
                onPrint();
            }
        });
        datePicker.show();
    }

    @Override
    public void onClear() {
        super.onClear();
        currentDate = null;
        datePicker = null;
    }

    private void onPrint() {
        edtInput.setText(dateFormat.format(currentDate));
    }
}
