package xyz.wbsite.wbui.widget.form;

import android.content.Context;
import android.content.res.TypedArray;
import android.os.Parcelable;
import android.support.annotation.Nullable;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.AttributeSet;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import xyz.wbsite.wbui.R;

public class FormInputView extends LinearLayout {

    protected String label;
    protected boolean required;
    protected boolean readOnly;

    protected TextView txtLabel;
    protected EditText edtInput;
    protected ImageView imgDelete;
    protected ImageView imgIcon;

    protected String code;
    protected String value;

    public FormInputView(Context context) {
        this(context, null);
    }

    public FormInputView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init(attrs);
    }

    public FormInputView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(attrs);
    }

    protected void init(AttributeSet attrs) {
        setSaveEnabled(false);
        // 获取自定义属性
        TypedArray a = getContext().obtainStyledAttributes(attrs, R.styleable.FormView);
        label = a.getString(R.styleable.FormView_label);
        required = a.getBoolean(R.styleable.FormView_required, false);
        readOnly = a.getBoolean(R.styleable.FormView_readOnly, false);
        a.recycle();
        // 获取自定义布局
        LayoutInflater.from(getContext()).inflate(R.layout.ui_form_input_view, this);
        txtLabel = findViewById(R.id.txtLabel);
        edtInput = findViewById(R.id.edtInput);
        imgDelete = findViewById(R.id.imgDelete);
        imgIcon = findViewById(R.id.imgIcon);
        txtLabel.setText(label);
        imgDelete.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                onClear();
            }
        });
        edtInput.setHint(R.string.form_input_hint);
        edtInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if (start == 0 && before == 0 && count == 0) {
                } else {
                    code = value = s.toString();
                    edtInput.setError(validData());
                    if (s.length() > 0) {
                        imgDelete.setVisibility(VISIBLE);
                    } else {
                        imgDelete.setVisibility(INVISIBLE);
                    }
                }
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });
        edtInput.setOnFocusChangeListener(new OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    edtInput.setError(validData());
                }
            }
        });
    }

    public String validData() {
        if (required && TextUtils.isEmpty(code)) {
            return label + "不能为空!";
        }
        return null;
    }

    public void onClear(){
        edtInput.setText("");
    }

    @Override
    protected void dispatchRestoreInstanceState(SparseArray<Parcelable> container) {

    }

}
