package xyz.wbsite.wbui.widget.form;


import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.res.TypedArray;
import android.net.Uri;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.qmuiteam.qmui.widget.dialog.QMUIBottomSheet;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import xyz.wbsite.activity.ImageViewerActivity;
import xyz.wbsite.activity.TakePhotoActivity;
import xyz.wbsite.base.BaseSPAActivity;
import xyz.wbsite.base.IActivityResult;
import xyz.wbsite.util.AnimationUtil;
import xyz.wbsite.util.Toaster;
import xyz.wbsite.wbui.R;

public class FormUploadView extends LinearLayout implements AdapterView.OnItemClickListener, AdapterView.OnItemLongClickListener {

    private TextView labelView;
    private GridView gridView;
    private List<Item> itemList = new ArrayList<>();//Uri为null的是添加图标
    private ItemAdapter itemAdapter = new ItemAdapter();
    private String label;
    private int minNum = 0;
    private int maxNum = 3;

    public FormUploadView(Context context) {
        super(context);
        init();
    }

    public FormUploadView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.FormView);
        label = a.getString(R.styleable.FormView_label);
        minNum = a.getInt(R.styleable.FormView_min_num, 0);
        maxNum = a.getInt(R.styleable.FormView_max_num, 3);
        a.recycle();
        init();
    }

    private void init() {
        View inflate = LayoutInflater.from(getContext()).inflate(R.layout.ui_form_upload_view, this);
        labelView = inflate.findViewById(R.id.label);
        labelView.setText(label);
        gridView = inflate.findViewById(R.id.gridView);
        gridView.setAdapter(itemAdapter);
        gridView.setOnItemClickListener(this);
        gridView.setOnItemLongClickListener(this);
        itemList.add(new Item(R.drawable.png_icon_add));
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        Item item = itemList.get(position);
        if (item.uri == null) {
            if (itemList.size() > maxNum) {
                Toaster.showToast("文件数量已达到最大");
                return;
            }
            new QMUIBottomSheet.BottomListSheetBuilder(getContext())
                    .addItem("拍照")
                    .addItem("相册")
                    .setOnSheetItemClickListener(new QMUIBottomSheet.BottomListSheetBuilder.OnSheetItemClickListener() {
                        @Override
                        public void onClick(QMUIBottomSheet dialog, View itemView, int position, String tag) {
                            dialog.dismiss();
                            Context context = getContext();
                            BaseSPAActivity activity = (BaseSPAActivity) context;
                            Intent intent = new Intent(activity, TakePhotoActivity.class);

                            switch (position) {
                                case 0:
                                    intent.putExtra(TakePhotoActivity.POSITION_PARAM, TakePhotoActivity.POSITION_CAPTURE);
                                    break;
                                case 1:
                                    intent.putExtra(TakePhotoActivity.POSITION_PARAM, TakePhotoActivity.POSITION_GALLERY);
                                    break;
                                case 2:
                                    intent.putExtra(TakePhotoActivity.POSITION_PARAM, TakePhotoActivity.POSITION_DOCUMENTS);
                                    break;
                            }

                            activity.startForResult(intent, new IActivityResult() {
                                @Override
                                public void onResult(int resultCode, Intent data) {
                                    if (resultCode == Activity.RESULT_OK && data != null) {
                                        Uri uri = data.getData();
                                        addItem(new Item(uri));
                                    }
                                }
                            });
                        }
                    }).build().show();
        } else {
            Context context = getContext();
            if (context instanceof BaseSPAActivity) {
                BaseSPAActivity activity = (BaseSPAActivity) context;
                Intent intent = new Intent(activity, ImageViewerActivity.class);
                intent.setData(itemList.get(position).uri);
                activity.startActivity(intent);
            }
        }
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> parent, final View view, final int i, long id) {
        Item item = itemList.get(i);
        if (item.uri != null) {
            new QMUIBottomSheet.BottomListSheetBuilder(getContext())
                    .addItem("删除")
                    .setOnSheetItemClickListener(new QMUIBottomSheet.BottomListSheetBuilder.OnSheetItemClickListener() {
                        @Override
                        public void onClick(QMUIBottomSheet dialog, View itemView, int position, String tag) {
                            dialog.dismiss();
                            switch (position) {
                                case 0: {
                                    // 移除前使用动画
                                    AnimationUtil.toFadeOut(view, 40 * 4, new Animation.AnimationListener() {
                                        @Override
                                        public void onAnimationStart(Animation animation) {

                                        }

                                        @Override
                                        public void onAnimationEnd(Animation animation) {
                                            removeItem(i);
                                        }

                                        @Override
                                        public void onAnimationRepeat(Animation animation) {

                                        }
                                    });
                                }
                                break;
                            }

                        }
                    }).build().show();
        }
        return true;
    }

    public void addItem(Item item) {
        if (itemList.get(itemList.size() - 1).uri == null) {
            itemList.add(itemList.size() - 1, item);
        } else {
            itemList.add(item);
        }
        itemAdapter.notifyDataSetChanged();
    }

    public void removeItem(int position) {
        itemList.remove(position);
        itemAdapter.notifyDataSetChanged();
    }

    /**
     * 显示添加图标
     */
    public void showAddIcon() {
        Item item = itemList.get(itemList.size() - 1);
        if (item.uri != null) {
            itemList.add(new Item(R.drawable.png_icon_add));
            itemAdapter.notifyDataSetChanged();
        }
    }

    /**
     * 隐藏添加图标
     */
    public void hideAddItem() {
        Item item = itemList.get(itemList.size() - 1);
        if (item.uri == null) {
            removeItem(itemList.size() - 1);
            itemAdapter.notifyDataSetChanged();
        }
    }

    class Item {
        private Uri uri;
        private int ressource;

        public Item(int ressource) {
            this.ressource = ressource;
        }

        public Item(Uri uri) {
            this.uri = uri;
        }
    }

    class ItemAdapter extends BaseAdapter {

        @Override
        public int getCount() {
            return itemList.size();
        }

        @Override
        public Object getItem(int i) {
            return itemList.get(i);
        }

        @Override
        public long getItemId(int i) {
            return i;
        }

        @Override
        public View getView(int i, View view, ViewGroup viewGroup) {
            View view_;
            ViewHolder holder;
            if (null == view) {
                view_ = LayoutInflater.from(getContext()).inflate(R.layout.ui_form_upload_view_item, null);
                holder = new ViewHolder(view_);
                view_.setTag(holder);
            } else {
                view_ = view;
                holder = (ViewHolder) view.getTag();
            }

            if (itemList.get(i).uri != null) {
                Glide.with(getContext()).load(itemList.get(i).uri).into(holder.imgIcon);
            } else {
                Glide.with(getContext()).load(itemList.get(i).ressource).into(holder.imgIcon);
            }
            return view_;
        }

        class ViewHolder {
            @BindView(R.id.imgIcon)
            ImageView imgIcon;

            ViewHolder(View view) {
                ButterKnife.bind(this, view);
            }
        }
    }
}
