package xyz.wbsite.wbui.widget.form;

import android.content.Context;
import android.content.res.TypedArray;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.view.View;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import cn.qqtheme.framework.picker.DateTimePicker;
import cn.qqtheme.framework.picker.TimePicker;
import xyz.wbsite.base.BaseSPAActivity;
import xyz.wbsite.wbui.R;

public class TIMEFormInputView extends FormInputView {
    private Date currentDate;
    private SimpleDateFormat dateFormat;

    private TimePicker datePicker;

    public TIMEFormInputView(Context context) {
        super(context);
    }

    public TIMEFormInputView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    @Override
    protected void init(AttributeSet attrs) {
        super.init(attrs);
        // 获取自定义属性
        TypedArray a = getContext().obtainStyledAttributes(attrs, R.styleable.FormView);
        String format_ = a.getString(R.styleable.FormView_format);
        if (format_ == null) {
            format_ = "HH:mm:ss";
        }
        dateFormat = new SimpleDateFormat(format_, Locale.CHINA);
        a.recycle();
        //初始化InputView
        edtInput.setEnabled(true);
        edtInput.setFocusable(false);
        edtInput.setKeyListener(null);
        edtInput.setHint("请选择时间");
        edtInput.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                onShowPicker();
            }
        });
    }

    private void onShowPicker() {
        if (currentDate == null) {
            currentDate = new Date();
        }

        Calendar instance = Calendar.getInstance();
        instance.setTime(currentDate);

        //初始化日期选择器
        datePicker = new TimePicker((BaseSPAActivity) getContext(), DateTimePicker.HOUR_24);
        datePicker.setSelectedItem(
                instance.get(Calendar.HOUR_OF_DAY),
                instance.get(Calendar.MINUTE));
        datePicker.setOnTimePickListener(new TimePicker.OnTimePickListener() {

            @Override
            public void onTimePicked(String hour, String minute) {
                Calendar instance = Calendar.getInstance();
                instance.setTime(currentDate);
                instance.set(Calendar.HOUR_OF_DAY, Integer.parseInt(hour));
                instance.set(Calendar.MINUTE, Integer.parseInt(minute));
                instance.set(Calendar.SECOND, 0);
                currentDate = instance.getTime();
                onPrint();
            }
        });
        datePicker.show();
    }

    @Override
    public void onClear() {
        super.onClear();
        currentDate = null;
        datePicker = null;
    }

    private void onPrint() {
        edtInput.setText(dateFormat.format(currentDate));
    }
}
